#include <lib/gui/elistbox.h>
#include <lib/gui/elistboxcontent.h>

eListbox::eListbox(eWidget *parent): eWidget(parent)
{
	setContent(new eListboxStringContent());
}

void eListbox::setContent(iListboxContent *content)
{
	m_content = content;
	invalidate();
	m_content->cursorHome();
	m_top = 0;
	m_selected = 0;
}

void eListbox::moveSelection(int dir)
{
		/* we need the old top/sel to see what we have to redraw */
	int oldtop = m_top;
	int oldsel = m_selected;
	
		/* first, move cursor */
	switch (dir)
	{
	case moveUp:
		m_content->cursorMove(-1);
		break;
	case moveDown:
		m_content->cursorMove(1);
			/* ok - we could have reached the end. we just go one back then. */
		if (!m_content->cursorValid())
			m_content->cursorMove(-1);
		break;
	case moveTop:
		m_content->cursorHome();
		m_top = 0; /* align with top, speeds up process */
		break;
	case moveEnd:
			/* move to last existing one ("end" is already invalid) */
		m_content->cursorEnd(); m_content->cursorMove(-1); 
		
		m_top = m_content->cursorGet() - m_items_per_page + 1;
		if (m_top < 0)
			m_top = 0;
		break;
	}
	
		/* note that we could be on an invalid cursor position, but we don't
		   care. this only happens on empty lists, and should have almost no
		   side effects. */
	
		/* now, look wether the current selection is out of screen */
	m_selected = m_content->cursorGet();
	if (m_selected < m_top)
	{
		m_top -= m_items_per_page;
		if (m_top < 0)
			m_top = 0;
	} else if (m_selected >= m_top + m_items_per_page)
	{
			/* m_top should be always valid here as it's selected */
		m_top += m_items_per_page;
	}
	
	if (m_top != oldtop)
		invalidate();
	else
	{
			/* redraw the old and newly selected */
		gRegion inv = eRect(0, m_itemheight * (m_selected-m_top), size().width(), m_itemheight);
		inv |= eRect(0, m_itemheight * (oldsel-m_top), size().width(), m_itemheight);
		
		invalidate(inv);
	}
}

int eListbox::event(int event, void *data, void *data2)
{
	switch (event)
	{
	case evtPaint:
	{
		ePtr<eWindowStyle> style;
		
		assert(m_content);
		recalcSize(); // move to event
		
		getStyle(style);
		
		if (!m_content)
			return 0;
		
		gPainter &painter = *(gPainter*)data2;
		
		m_content->cursorSave();
		m_content->cursorMove(m_top - m_selected);
		
		for (int y = 0, i = 0; i < m_items_per_page; y += m_itemheight, ++i)
		{
			m_content->paint(painter, *style, ePoint(0, y), m_selected == m_content->cursorGet());
			m_content->cursorMove(+1);
		}
		
		m_content->cursorRestore();
		
		return 0;
	}
	default:
		return eWidget::event(event, data, data2);
	}
}

void eListbox::recalcSize()
{
	eDebug("recalc size");
	m_itemheight = 20;
	m_content->setSize(eSize(size().width(), m_itemheight));
	m_items_per_page = size().height() / m_itemheight;
	eDebug("done!");
}
